@@ -1,5 +1,6 @@
 import os
 import re
+import logging
 
 import numpy as np
 import pandas as pd
@@ -16,7 +17,7 @@ from .metrics.cluster import clust_compute
 #     from .metrics.dim_red import dr_compute
 # except ImportError:
 #     from metrics.dim_red import dr_compute
-
+#
 # try:
 #     from . import checkatlas, folders
 # except ImportError:
@@ -40,21 +41,30 @@ OBS_CLUSTERS = [
     "orig.ident",
 ]
 
+OBS_QC = [
+    "n_genes_by_counts",
+    "total_counts",
+    "pct_counts_mt",
+    "pct_counts_ribo",
+    "percent.mito",
+    "percent.ribo",
+    "dropouts",
+    "nCount_SCT",
+    "nFeature_SCT",
+    "nCount_RNA",
+    "nFeature_RNA",
+    "nCount_HTO",
+    "nFeature_HTO",
+    "n_genes",
+    "n_genes_by_counts",
+    "total_counts",
+    "total_counts_mt",
+    "pct_counts_mt",
+    "total_counts_ribo",
+    "pct_counts_ribo",
+]
 
-def list_atlases(path) -> list:
-    """
-    List all atlases files in the path
-    Detect .rds, .h5, .h5ad
-    :param path:
-    :return: List of files
-    """
-    atlas_list = list()
-    for root, dirs, files in os.walk(path):
-        for file in files:
-            for extension in checkatlas.EXTENSIONS:
-                if file.endswith(extension):
-                    atlas_list.append(os.path.join(root, file))
-    return atlas_list
+logger = logging.getLogger("checkatlas")
 
 
 def convert_atlas(atlas_path, atlas_name) -> None:
@@ -63,7 +73,7 @@ def convert_atlas(atlas_path, atlas_name) -> None:
     :param atlas_path:
     :return:
     """
-    print("Convert Seurat object to Scanpy: ", atlas_path)
+    logger.info(f"Convert Seurat object to Scanpy: {atlas_path}")
     rscript_cmd = (
         "Rscript "
         + checkatlas.RSCRIPT
@@ -72,7 +82,7 @@ def convert_atlas(atlas_path, atlas_name) -> None:
         + " "
         + atlas_name
     )
-    print(rscript_cmd)
+    logger.debug(f"Run: {rscript_cmd}")
     os.system(rscript_cmd)
 
 
@@ -82,7 +92,7 @@ def clean_scanpy_atlas(adata, atlas_info) -> bool:
     :param adata:
     :return:
     """
-    print("Clean scanpy:" + atlas_info[0])
+    logger.debug(f"Clean scanpy: {atlas_info[0]}")
     # If OBS_CLUSTERS are present and in int32 -> be sure to
     # transform them in categorical
     for obs_key in adata.obs_keys():
@@ -93,7 +103,21 @@ def clean_scanpy_atlas(adata, atlas_info) -> bool:
     return adata
 
 
-def get_viable_obs(adata):
+def get_viable_obs_qc(adata):
+    """
+    Search in obs_keys a match to OBS_QC values
+    Extract sorted obs_keys in same order then OBS_QC
+    :param adata:
+    :return:
+    """
+    obs_keys = list()
+    for obs_key in adata.obs_keys():
+        if obs_key in OBS_QC:
+            obs_keys.append(obs_key)
+    return obs_keys
+
+
+def get_viable_obs_annot(adata):
     """
     Search in obs_keys a match to OBS_CLUSTERS values
     Extract sorted obs_keys in same order then OBS_CLUSTERS
@@ -106,12 +130,10 @@ def get_viable_obs(adata):
             if obs_key_celltype in obs_key:
                 if type(adata.obs[obs_key].dtype) == pd.CategoricalDtype:
                     obs_keys.append(obs_key)
-    # ### obs are sorted to have cell_type first
-    # (! Need to fix that accordingly)
     return sorted(obs_keys)
 
 
-def create_summary_table(adata, atlas_path, atlas_info, path) -> None:
+def create_summary_table(adata, atlas_path, atlas_info, args) -> None:
     """
     Create a table with all interesting variables
     :param adata:
@@ -120,10 +142,11 @@ def create_summary_table(adata, atlas_path, atlas_info, path) -> None:
     :return:
     """
     atlas_name = atlas_info[0]
+    logger.debug(f"Create Summary table for {atlas_name}")
     atlas_file_type = atlas_info[1]
     atlas_extension = atlas_info[2]
     csv_path = os.path.join(
-        folders.get_folder(path, folders.SUMMARY),
+        folders.get_folder(args.path, folders.SUMMARY),
         atlas_name + checkatlas.SUMMARY_EXTENSION,
     )
     # Create summary table
@@ -136,7 +159,6 @@ def create_summary_table(adata, atlas_path, atlas_info, path) -> None:
         "File_extension",
         "File_path",
     ]
-    print("Run summary")
     df_summary = pd.DataFrame(index=[atlas_name], columns=header)
     df_summary["AtlasFileType"][atlas_name] = atlas_file_type
     df_summary["NbCells"][atlas_name] = adata.n_obs
@@ -144,11 +166,11 @@ def create_summary_table(adata, atlas_path, atlas_info, path) -> None:
     df_summary["AnnData.raw"][atlas_name] = adata.raw is not None
     df_summary["AnnData.X"][atlas_name] = adata.X is not None
     df_summary["File_extension"][atlas_name] = atlas_extension
-    df_summary["File_path"][atlas_name] = atlas_path.replace(path, "")
-    df_summary.to_csv(csv_path, index=False)
+    df_summary["File_path"][atlas_name] = atlas_path.replace(args.path, "")
+    df_summary.to_csv(csv_path, index=False, sep="\t")
 
 
-def create_anndata_table(adata, atlas_path, atlas_info, path) -> None:
+def create_anndata_table(adata, atlas_path, atlas_info, args) -> None:
     """
     Create a table with all AnnData arguments
     :param adata:
@@ -157,8 +179,9 @@ def create_anndata_table(adata, atlas_path, atlas_info, path) -> None:
     :return:
     """
     atlas_name = atlas_info[0]
+    logger.debug(f"Create Adata table for {atlas_name}")
     csv_path = os.path.join(
-        folders.get_folder(path, folders.ANNDATA),
+        folders.get_folder(args.path, folders.ANNDATA),
         atlas_name + checkatlas.ADATA_EXTENSION,
     )
     # Create AnnData table
@@ -190,10 +213,10 @@ def create_anndata_table(adata, atlas_path, atlas_info, path) -> None:
     df_summary["uns"][atlas_name] = (
         "<code>" + "</code><br><code>".join(list(adata.uns_keys())) + "</code>"
     )
-    df_summary.to_csv(csv_path, index=False, quoting=False)
+    df_summary.to_csv(csv_path, index=False, quoting=False, sep="\t")
 
 
-def create_qc_plots(adata, atlas_path, atlas_info, path) -> None:
+def create_qc_tables(adata, atlas_path, atlas_info, args) -> None:
     """
     Display the atlas QC
     Search for the OBS variable which correspond to the toal_RNA, total_UMI,
@@ -205,9 +228,41 @@ def create_qc_plots(adata, atlas_path, atlas_info, path) -> None:
     :return:
     """
     atlas_name = atlas_info[0]
-    sc.settings.figdir = folders.get_workingdir(path)
-    qc_path = os.sep + atlas_name + checkatlas.QC_EXTENSION
-    print("Calc QC")
+    qc_path = os.path.join(
+        folders.get_folder(args.path, folders.QC),
+        atlas_name + checkatlas.QC_EXTENSION,
+    )
+    logger.debug(f"Create QC tables for {atlas_name}")
+    # mitochondrial genes
+    adata.var["mt"] = adata.var_names.str.startswith("MT-")
+    # ribosomal genes
+    adata.var["ribo"] = adata.var_names.str.startswith(("RPS", "RPL"))
+    sc.pp.calculate_qc_metrics(
+        adata,
+        qc_vars=["mt", "ribo"],
+        percent_top=None,
+        log1p=False,
+        inplace=True,
+    )
+    df_annot = adata.obs[get_viable_obs_qc(adata)]
+    df_annot.to_csv(qc_path, index=False, quoting=False, sep="\t")
+
+
+def create_qc_plots(adata, atlas_path, atlas_info, args) -> None:
+    """
+    Display the atlas QC
+    Search for the OBS variable which correspond to the toal_RNA, total_UMI,
+     MT_ratio, RT_ratio
+    :param path:
+    :param adata:
+    :param atlas_name:
+    :param atlas_path:
+    :return:
+    """
+    atlas_name = atlas_info[0]
+    sc.settings.figdir = folders.get_workingdir(args.path)
+    qc_path = os.sep + atlas_name + checkatlas.QC_FIG_EXTENSION
+    logger.debug(f"Create QC violin plot for {atlas_name}")
     # mitochondrial genes
     adata.var["mt"] = adata.var_names.str.startswith("MT-")
     # ribosomal genes
@@ -234,7 +289,7 @@ def create_qc_plots(adata, atlas_path, atlas_info, path) -> None:
     )
 
 
-def create_umap_fig(adata, atlas_path, atlas_info, path) -> None:
+def create_umap_fig(adata, atlas_path, atlas_info, args) -> None:
     """
     Display the UMAP of celltypes
     Search for the OBS variable which correspond to the celltype annotation
@@ -247,20 +302,20 @@ def create_umap_fig(adata, atlas_path, atlas_info, path) -> None:
     atlas_name = atlas_info[0]
     # Search if tsne reduction exists
     r = re.compile(".*umap*.")
-    print(len(list(filter(r.match, adata.obsm_keys()))))
     if len(list(filter(r.match, adata.obsm_keys()))) > 0:
+        logger.debug(f"Create UMAP figure for {atlas_name}")
         # Setting up figures directory
-        sc.settings.figdir = folders.get_workingdir(path)
+        sc.settings.figdir = folders.get_workingdir(args.path)
         umap_path = os.sep + atlas_name + checkatlas.UMAP_EXTENSION
         # Exporting umap
-        obs_keys = get_viable_obs(adata)
+        obs_keys = get_viable_obs_annot(adata)
         if len(obs_keys) != 0:
             sc.pl.umap(adata, color=obs_keys[0], show=False, save=umap_path)
         else:
             sc.pl.umap(adata, show=False, save=umap_path)
 
 
-def create_tsne_fig(adata, atlas_path, atlas_info, path) -> None:
+def create_tsne_fig(adata, atlas_path, atlas_info, args) -> None:
     """
     Display the TSNE of celltypes
     Search for the OBS variable which correspond to the celltype annotation
@@ -274,18 +329,21 @@ def create_tsne_fig(adata, atlas_path, atlas_info, path) -> None:
     atlas_name = atlas_info[0]
     r = re.compile(".*tsne*.")
     if len(list(filter(r.match, adata.obsm_keys()))) > 0:
+        logger.debug(f"Create t-SNE figure for {atlas_name}")
         # Setting up figures directory
-        sc.settings.figdir = sc.settings.figdir = folders.get_workingdir(path)
+        sc.settings.figdir = sc.settings.figdir = folders.get_workingdir(
+            args.path
+        )
         tsne_path = os.sep + atlas_name + checkatlas.TSNE_EXTENSION
         # Exporting tsne
-        obs_keys = get_viable_obs(adata)
+        obs_keys = get_viable_obs_annot(adata)
         if len(obs_keys) != 0:
             sc.pl.tsne(adata, color=obs_keys[0], show=False, save=tsne_path)
         else:
             sc.pl.tsne(adata, show=False, save=tsne_path)
 
 
-def metric_cluster(adata, atlas_path, atlas_info, path) -> None:
+def metric_cluster(adata, atlas_path, atlas_info, args) -> None:
     """
     Main function of checkatlas
     For every atlas create summary tables with all attributes of the atlas
@@ -295,21 +353,29 @@ def metric_cluster(adata, atlas_path, atlas_info, path) -> None:
     """
     atlas_name = atlas_info[0]
     csv_path = os.path.join(
-        folders.get_folder(path, folders.CLUSTER),
+        folders.get_folder(args.path, folders.CLUSTER),
         atlas_name + checkatlas.METRIC_CLUSTER_EXTENSION,
     )
     header = ["Sample", "obs", "Silhouette", "Davies-Bouldin"]
     df_cluster = pd.DataFrame(columns=header)
-    obs_keys = get_viable_obs(adata)
+    obs_keys = get_viable_obs_annot(adata)
+    if len(obs_keys) > 0:
+        logger.debug(f"Calc Clustering metrics for {atlas_name}")
+    else:
+        logger.debug(f"No viable obs_key was found for {atlas_name}")
     for obs_key in obs_keys:
         # Need more than one sample to calculate metric
         annotations = adata.obs[obs_key]
         if len(annotations.cat.categories) != 1:
             silhouette = 1
-            print("Calc Silhouette for " + atlas_name, obs_key)
+            logger.debug(
+                f"Calc Silhouette for {atlas_name} with obs {obs_key}"
+            )
             silhouette = clust_compute.silhouette(adata, obs_key, "X_umap")
             daviesb = -1
-            print("Calc Davies Bouldin for " + atlas_name, obs_key)
+            logger.debug(
+                f"Calc Davies Bouldin for {atlas_name} with obs {obs_key}"
+            )
             daviesb = clust_compute.davies_bouldin(adata, obs_key, "X_umap")
             df_line = pd.DataFrame(
                 {
@@ -323,10 +389,10 @@ def metric_cluster(adata, atlas_path, atlas_info, path) -> None:
                 [df_cluster, df_line], ignore_index=True, axis=0
             )
     if len(df_cluster) != 0:
-        df_cluster.to_csv(csv_path, index=False)
+        df_cluster.to_csv(csv_path, index=False, sep="\t")
 
 
-def metric_annot(adata, atlas_path, atlas_info, path) -> None:
+def metric_annot(adata, atlas_path, atlas_info, args) -> None:
     """
     Main function of checkatlas
     For every atlas create summary tables with all attributes of the atlas
@@ -336,12 +402,12 @@ def metric_annot(adata, atlas_path, atlas_info, path) -> None:
     """
     atlas_name = atlas_info[0]
     csv_path = os.path.join(
-        folders.get_folder(path, folders.ANNOTATION),
+        folders.get_folder(args.path, folders.ANNOTATION),
         atlas_name + checkatlas.METRIC_ANNOTATION_EXTENSION,
     )
     header = ["Sample", "obs", "Rand"]
     df_annot = pd.DataFrame(columns=header)
-    obs_keys = get_viable_obs(adata)
+    obs_keys = get_viable_obs_annot(adata)
     if len(obs_keys) != 0:
         ref_obs = obs_keys[0]
         for i in range(1, len(obs_keys)):
@@ -349,9 +415,9 @@ def metric_annot(adata, atlas_path, atlas_info, path) -> None:
             # Need more than one sample to calculate metric
             annotations = adata.obs[obs_key]
             if len(annotations.cat.categories) != 1:
-                print(
-                    "NOT WORKING YET - Calc Rand Index for " + atlas_name,
-                    obs_key,
+                logger.debug(
+                    f"NOT WORKING YET - Calc Rand Index "
+                    f"for {atlas_name} with obs {obs_key}"
                 )
                 rand = -1
                 # ##rand = clust_compute.rand(adata, obs_key, ref_obs)
@@ -367,10 +433,10 @@ def metric_annot(adata, atlas_path, atlas_info, path) -> None:
                     [df_annot, df_line], ignore_index=True, axis=0
                 )
         if len(df_annot) != 0:
-            df_annot.to_csv(csv_path, index=False)
+            df_annot.to_csv(csv_path, index=False, sep="\t")
 
 
-def metric_dimred(adata, atlas_path, atlas_info, path) -> None:
+def metric_dimred(adata, atlas_path, atlas_info, args) -> None:
     """
     Main function of checkatlas
     For every atlas create summary tables with all attributes of the atlas
@@ -380,14 +446,15 @@ def metric_dimred(adata, atlas_path, atlas_info, path) -> None:
     """
     atlas_name = atlas_info[0]
     csv_path = os.path.join(
-        folders.get_folder(path, folders.DIMRED),
+        folders.get_folder(args.path, folders.DIMRED),
         atlas_name + checkatlas.METRIC_DIMRED_EXTENSION,
     )
     header = ["Sample", "obs", "Kruskal"]
     df_dimred = pd.DataFrame(columns=header)
     for obsm_key in adata.obsm_keys():
-        print(
-            "NOT WORKING YET - Calc Kruskal Stress for " + atlas_name, obsm_key
+        logger.debug(
+            f"NOT WORKING YET - Calc Rand Index "
+            f"for {atlas_name} with obs {obsm_key}"
         )
         kruskal = -1
         # kruskal = dr_compute.kruskal_stress(adata, obsm_key)
@@ -400,4 +467,4 @@ def metric_dimred(adata, atlas_path, atlas_info, path) -> None:
         )
         df_dimred = pd.concat([df_dimred, df_line], ignore_index=True, axis=0)
     if len(df_dimred) != 0:
-        df_dimred.to_csv(csv_path, index=False)
+        df_dimred.to_csv(csv_path, index=False, sep="\t")
