@@ -1,9 +1,11 @@
 import os
+import logging
 
 working_dir = "checkatlas_files"
 SUMMARY = "summary"
 ANNDATA = "adata"
-QC = "violin"
+QC = "qc"
+QC_FIG = "violin"
 UMAP = "umap"
 TSNE = "tsne"
 CLUSTER = "cluster"
@@ -15,6 +17,7 @@ dict_folder = {
     SUMMARY: SUMMARY,
     ANNDATA: ANNDATA,
     QC: QC,
+    QC_FIG: QC_FIG,
     UMAP: UMAP,
     TSNE: TSNE,
     CLUSTER: CLUSTER,
@@ -23,6 +26,8 @@ dict_folder = {
     SPECI: SPECI,
 }
 
+logger = logging.getLogger("checkatlas")
+
 
 def get_workingdir(path):
     return os.path.join(path, working_dir)
@@ -33,7 +38,12 @@ def get_folder(path, key_folder):
 
 
 def checkatlas_folders(path):
-    print("Check if checkatlas folders exist")
+    """
+    Check in path if the different checkatlas folders exists.<br>
+    Create them if needed.
+    :param path:
+    :return: None
+    """
     global_path = get_workingdir(path)
     if not os.path.exists(global_path):
         os.mkdir(global_path)
@@ -41,4 +51,5 @@ def checkatlas_folders(path):
     for key_folder in dict_folder.keys():
         temp_path = os.path.join(global_path, key_folder)
         if not os.path.exists(temp_path):
+            logger.debug(f"Create folder: {temp_path}")
             os.mkdir(temp_path)
